;;; mosaic-fetch-url.el     tells NCSA Mosaic to get a given URL
;;;
;;; Copyright (c)1993 Darrell Kindred <dkindred@cs.cmu.edu>
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;; This pkg provides the mosaic-fetch-url function, which instructs a 
;;; running Mosaic process to fetch the document corresponding to the
;;; given URL.
;;;
;;; Note:
;;;    The process id for a running Mosaic must be stored in mosaic-pid-file
;;;    (default ~/.mosaicpid).  In Mosaic 2.0pre5 and later, this happens by
;;;    default.  
;;;
;;; Changes:
;;;    28-Aug-94   switched to using /bin/kill instead of signal-process,
;;;                since HPUX uses a different signal number for SIGUSR1.

(defvar mosaic-pid-file (expand-file-name "~/.mosaicpid")
 "File containing the pid of a Mosaic process to communicate with.")

(defvar mosaic-remote-control-file "/tmp/Mosaic.%d"
  "File to write into to send commands to a running Mosaic process.  
A %d will expand to the pid of the Mosaic process.")

(defvar mosaic-kill-program "/bin/kill"
  "Program to use to send a USR1 signal to a process.  Must accept -USR1 and 
a process id as arguments.  If this is nil, the signal-process function
will be used instead (if available), sending mosaic-nudge-signal.")

(defvar mosaic-nudge-signal 30
  "Signal number to tell a Mosaic process to read mosaic-remote-control-file
(should be SIGUSR1).  Too bad process-send-signal doesn't understand signal 
names.  This is only used if mosaic-kill-program is nil.")

(defvar mosaic-use-new-window t
  "*If non-nil, mosaic-fetch-url will cause the document to be displayed
in a new Mosaic window.  Otherwise, the current Mosaic window is used.")

(defvar mosaic-quiet nil
  "*If non-nil, don't print the message saying that Mosaic has been
asked to find a document.")

(defun mosaic-fetch-url (&optional buf b e pfx url)
  "Instruct a running Mosaic process to fetch a document with 
the given URL.  If mosaic-use-new-window is non-nil, the document
will be displayed in a new window; otherwise, it will be displayed
in the current window.  If PFX is >1, prompt the user using the given
URL as the default.

The process id for a running Mosaic must be stored in
mosaic-pid-file (default ~/.mosaicpid).  In Mosaic 2.0 and
later, this happens by default."
  (interactive)
  (if (or (and pfx (> pfx 1))
	  (null url))
      (setq url (read-from-minibuffer "URL: " url)))
  (if (not (stringp url))
      (error "mosaic-fetch-url: bad argument (expected a string)"))
  (save-excursion
    (let ((pid  (mosaic-get-pid)))
      (mosaic-write-fetch-command url pid)
      (mosaic-wakeup-mosaic-process pid))))
      
(defun mosaic-get-pid ()
  (let ((b (generate-new-buffer "*mosaic-pid*")))
    (set-buffer b)
    (insert-file-contents mosaic-pid-file)
    (let ((pid  (string-to-int (buffer-substring (point-min) (point-max)))))
      (kill-buffer b)
      (if (or (< pid 2) (> pid 30000))
	  (error "bad Mosaic pid: %d" pid))
      pid)))

;; write the appropriate command to a file which mosaic will read
(defun mosaic-write-fetch-command (url pid)
  (let ((b (generate-new-buffer "*mosaic-url*")))
    (set-buffer b)
    (if mosaic-use-new-window
	(insert "newwin\n")
      (insert "goto\n"))
    (insert url "\n")
    (write-region (point-min) (point-max)
		  (format mosaic-remote-control-file pid)
		  nil 'foo)
    (kill-buffer b)))

;; wake up a running Mosaic process so it will read the command file
;; we just wrote to mosaic-remote-control-file
(defun mosaic-wakeup-mosaic-process (pid)
  ;; unfortunately there's no good way to find out whether 
  ;; we successfully signalled Mosaic under version 18 emacs
  ;; because signal-process doesn't exist and call-process doesn't
  ;; provide exit status.
  (cond (mosaic-kill-program
	 (let ((status (call-process mosaic-kill-program nil nil nil "-USR1" 
				     (int-to-string pid))))
	   (if (and (integerp status)
		    (not (= 0 status)))
	       (error "Can't signal Mosaic process (pid %d).  Does it exist?"
		      pid))))
	((fboundp 'signal-process)
	 ;; signal-process returns the return value from kill(), 
	 ;; although this is not documented.
	 (if (< (signal-process pid mosaic-nudge-signal) 0)
	     (error "Can't signal Mosaic process (pid %d).  Does it exist?"
		    pid)))
	(t
	 (error "signal-process doesn't exist and mosaic-kill-program is nil"))
	)
  (if (not mosaic-quiet)
      (message "Asked Mosaic for %s" url)))

(provide 'mosaic-fetch-url)
